@@ -407,6 +407,8 @@ _pango_core_text_font_description_from_ct_font_descriptor (CTFontDescriptorRef d
   char *family_name;
   char *style_name;
   PangoFontDescription *font_desc;
+  CFNumberRef cf_number;
+  CGFloat pointsize;
 
   font_desc = pango_font_description_new ();
 
@@ -419,6 +421,11 @@ _pango_core_text_font_description_from_ct_font_descriptor (CTFontDescriptorRef d
   pango_font_description_set_family (font_desc, family_name);
   g_free (family_name);
 
+  /* Size (if we have one) */
+  cf_number = CTFontDescriptorCopyAttribute (desc, kCTFontSizeAttribute);
+  if (cf_number != NULL && CFNumberGetValue (cf_number, kCFNumberCGFloatType, &pointsize))
+    pango_font_description_set_size (font_desc, (pointsize / (96./72.)) * 1024);
+
   /* Weight */
   pango_font_description_set_weight (font_desc,
                                      ct_font_descriptor_get_weight (desc));
@@ -849,7 +856,7 @@ get_scaled_size (PangoCoreTextFontMap       *fontmap,
                  const PangoFontDescription *desc)
 {
   double size = pango_font_description_get_size (desc);
-  PangoMatrix *matrix = pango_context_get_matrix (context);
+  const PangoMatrix *matrix = pango_context_get_matrix (context);
   double scale_factor = pango_matrix_get_font_scale_factor (matrix);
   
   if (!pango_font_description_get_size_is_absolute(desc))
@@ -1422,7 +1429,7 @@ pango_core_text_font_map_load_fontset (PangoFontMap               *fontmap,
   /* Cannot use pango_core_text_fontset_key_free() here */
   pango_font_description_free (key.desc);
 
-  return g_object_ref (fontset);
+  return g_object_ref (PANGO_FONTSET (fontset));
 }
 
 static void
