@@ -125,9 +125,6 @@ test_line_height (void)
   g_object_unref (context);
 }
 
-#if 0
-/* These tests fail since I had to revert 20ec670e124e446107
- */
 static void
 test_line_height2 (void)
 {
@@ -195,6 +192,12 @@ test_run_height (void)
   PangoLayoutIter *iter;
   PangoRectangle logical1, logical2;
 
+  if (strcmp (G_OBJECT_TYPE_NAME (pango_cairo_font_map_get_default ()), "PangoCairoCoreTextFontMap") == 0)
+    {
+      g_test_skip ("This test fails on macOS and needs debugging");
+      return;
+    }
+
   context = pango_font_map_create_context (pango_cairo_font_map_get_default ());
   layout = pango_layout_new (context);
   pango_layout_set_text (layout, "one", -1);
@@ -240,6 +243,12 @@ test_cursor_height2 (void)
   PangoLayout *layout;
   PangoRectangle strong1, strong2;
 
+  if (strcmp (G_OBJECT_TYPE_NAME (pango_cairo_font_map_get_default ()), "PangoCairoCoreTextFontMap") == 0)
+    {
+      g_test_skip ("This test fails on macOS and needs debugging");
+      return;
+    }
+
   context = pango_font_map_create_context (pango_cairo_font_map_get_default ());
   layout = pango_layout_new (context);
   pango_layout_set_text (layout, "one", -1);
@@ -255,7 +264,6 @@ test_cursor_height2 (void)
   g_object_unref (layout);
   g_object_unref (context);
 }
-#endif
 
 static void
 test_attr_list_update (void)
@@ -630,6 +638,12 @@ test_empty_line_height (void)
   cairo_font_options_t *options;
   int hint;
 
+  if (strcmp (G_OBJECT_TYPE_NAME (pango_cairo_font_map_get_default ()), "PangoCairoCoreTextFontMap") == 0)
+    {
+      g_test_skip ("This test fails on macOS and needs debugging");
+      return;
+    }
+
   context = pango_font_map_create_context (pango_cairo_font_map_get_default ());
 
   for (hint = CAIRO_HINT_METRICS_OFF; hint <= CAIRO_HINT_METRICS_ON; hint++)
@@ -661,6 +675,85 @@ test_empty_line_height (void)
   g_object_unref (context);
 }
 
+static void
+test_gravity_metrics (void)
+{
+  PangoFontMap *map;
+  PangoContext *context;
+  PangoFontDescription *desc;
+  PangoFont *font;
+  PangoGlyph glyph;
+  PangoGravity gravity;
+  PangoRectangle ink[4];
+  PangoRectangle log[4];
+
+  map = pango_cairo_font_map_get_default ();
+  context = pango_font_map_create_context (map);
+
+  desc = pango_font_description_from_string ("Cantarell 64");
+
+  glyph = 1; /* A */
+
+  for (gravity = PANGO_GRAVITY_SOUTH; gravity <= PANGO_GRAVITY_WEST; gravity++)
+    {
+      pango_font_description_set_gravity (desc, gravity);
+      font = pango_font_map_load_font (map, context, desc);
+      pango_font_get_glyph_extents (font, glyph, &ink[gravity], &log[gravity]);
+      g_object_unref (font);
+    }
+
+  g_assert_cmpint (ink[PANGO_GRAVITY_EAST].width, ==, ink[PANGO_GRAVITY_SOUTH].height);
+  g_assert_cmpint (ink[PANGO_GRAVITY_EAST].height, ==, ink[PANGO_GRAVITY_SOUTH].width);
+  g_assert_cmpint (ink[PANGO_GRAVITY_NORTH].width, ==, ink[PANGO_GRAVITY_SOUTH].width);
+  g_assert_cmpint (ink[PANGO_GRAVITY_NORTH].height, ==, ink[PANGO_GRAVITY_SOUTH].height);
+  g_assert_cmpint (ink[PANGO_GRAVITY_WEST].width, ==, ink[PANGO_GRAVITY_SOUTH].height);
+  g_assert_cmpint (ink[PANGO_GRAVITY_WEST].height, ==, ink[PANGO_GRAVITY_SOUTH].width);
+
+  g_assert_cmpint (log[PANGO_GRAVITY_SOUTH].width, ==, - log[PANGO_GRAVITY_NORTH].width);
+  g_assert_cmpint (log[PANGO_GRAVITY_EAST].width, ==, - log[PANGO_GRAVITY_WEST].width);
+
+  pango_font_description_free (desc);
+  g_object_unref (context);
+}
+
+static void
+test_transform_rectangle (void)
+{
+  PangoMatrix matrix = PANGO_MATRIX_INIT;
+  PangoRectangle rect;
+  PangoRectangle rect2;
+
+  rect = rect2 = (PangoRectangle) { 10 * PANGO_SCALE, 20 * PANGO_SCALE, 30 * PANGO_SCALE, 40 * PANGO_SCALE };
+  pango_matrix_transform_rectangle (&matrix, &rect2);
+
+  g_assert_cmpint (rect2.x, ==, rect.x);
+  g_assert_cmpint (rect2.y, ==, rect.y);
+  g_assert_cmpint (rect2.width, ==, rect.width);
+  g_assert_cmpint (rect2.height, ==, rect.height);
+
+  matrix = (PangoMatrix) PANGO_MATRIX_INIT;
+  pango_matrix_translate (&matrix, 10, 20);
+
+  pango_matrix_transform_rectangle (&matrix, &rect2);
+
+  g_assert_cmpint (rect2.x, ==, rect.x + 10 * PANGO_SCALE);
+  g_assert_cmpint (rect2.y, ==, rect.y + 20 * PANGO_SCALE);
+  g_assert_cmpint (rect2.width, ==, rect.width);
+  g_assert_cmpint (rect2.height, ==, rect.height);
+
+  rect2 = rect;
+
+  matrix = (PangoMatrix) PANGO_MATRIX_INIT;
+  pango_matrix_rotate (&matrix, -90);
+
+  pango_matrix_transform_rectangle (&matrix, &rect2);
+
+  g_assert_cmpint (rect2.x, ==, - (rect.y + rect.height));
+  g_assert_cmpint (rect2.y, ==, rect.x);
+  g_assert_cmpint (rect2.width, ==, rect.height);
+  g_assert_cmpint (rect2.height, ==, rect.width);
+}
+
 int
 main (int argc, char *argv[])
 {
@@ -672,11 +765,11 @@ main (int argc, char *argv[])
   g_test_add_func ("/layout/short-string-crash", test_short_string_crash);
   g_test_add_func ("/language/emoji-crash", test_language_emoji_crash);
   g_test_add_func ("/layout/line-height", test_line_height);
-  //g_test_add_func ("/layout/line-height2", test_line_height2);
-  //g_test_add_func ("/layout/line-height3", test_line_height3);
-  //g_test_add_func ("/layout/run-height", test_run_height);
-  //g_test_add_func ("/layout/cursor-height", test_cursor_height);
-  //g_test_add_func ("/layout/cursor-height2", test_cursor_height2);
+  g_test_add_func ("/layout/line-height2", test_line_height2);
+  g_test_add_func ("/layout/line-height3", test_line_height3);
+  g_test_add_func ("/layout/run-height", test_run_height);
+  g_test_add_func ("/layout/cursor-height", test_cursor_height);
+  g_test_add_func ("/layout/cursor-height2", test_cursor_height2);
   g_test_add_func ("/attr-list/update", test_attr_list_update);
   g_test_add_func ("/misc/version-info", test_version_info);
   g_test_add_func ("/misc/is-zerowidth", test_is_zero_width);
@@ -692,6 +785,8 @@ main (int argc, char *argv[])
   g_test_add_func ("/layout/index-to-x", test_index_to_x);
   g_test_add_func ("/layout/extents", test_extents);
   g_test_add_func ("/layout/empty-line-height", test_empty_line_height);
+  g_test_add_func ("/layout/gravity-metrics", test_gravity_metrics);
+  g_test_add_func ("/matrix/transform-rectangle", test_transform_rectangle);
 
   return g_test_run ();
 }
