@@ -149,17 +149,43 @@ pangocairo_view_destroy_surface (gpointer instance,
 }
 
 enum {
-  ANNOTATE_GRAVITY_ROOF      =   1,
-  ANNOTATE_BLOCK_PROGRESSION =   2,
-  ANNOTATE_BASELINES         =   4,
-  ANNOTATE_LAYOUT_EXTENTS    =   8,
-  ANNOTATE_LINE_EXTENTS      =  16,
-  ANNOTATE_RUN_EXTENTS       =  32,
-  ANNOTATE_CLUSTER_EXTENTS   =  64,
-  ANNOTATE_CHAR_EXTENTS      = 128,
-  ANNOTATE_LAST              = 256,
+  ANNOTATE_GRAVITY_ROOF      =    1,
+  ANNOTATE_BLOCK_PROGRESSION =    2,
+  ANNOTATE_BASELINES         =    4,
+  ANNOTATE_LAYOUT_EXTENTS    =    8,
+  ANNOTATE_LINE_EXTENTS      =   16,
+  ANNOTATE_RUN_EXTENTS       =   32,
+  ANNOTATE_CLUSTER_EXTENTS   =   64,
+  ANNOTATE_CHAR_EXTENTS      =  128,
+  ANNOTATE_GLYPH_EXTENTS     =  256,
+  ANNOTATE_CARET_POSITIONS   =  512,
+  ANNOTATE_CARET_SLOPE       = 1024,
+  ANNOTATE_LAST              = 2048,
 };
 
+static struct {
+  int value;
+  const char *name;
+  const char *short_name;
+} annotate_options[] = {
+  { ANNOTATE_GRAVITY_ROOF,      "gravity-roof",      "gravity" },
+  { ANNOTATE_BLOCK_PROGRESSION, "block-progression", "progression" },
+  { ANNOTATE_BASELINES,         "baselines",         "baselines" },
+  { ANNOTATE_LAYOUT_EXTENTS,    "layout-extents",    "layout" },
+  { ANNOTATE_LINE_EXTENTS,      "line-extents",      "line" },
+  { ANNOTATE_RUN_EXTENTS,       "run-extents",       "run" },
+  { ANNOTATE_CLUSTER_EXTENTS,   "cluster-extents",   "cluster" },
+  { ANNOTATE_CHAR_EXTENTS,      "char-extents",      "char" },
+  { ANNOTATE_GLYPH_EXTENTS,     "glyph-extents",     "glyph" },
+  { ANNOTATE_CARET_POSITIONS,   "caret-positions",   "caret" },
+  { ANNOTATE_CARET_SLOPE,       "caret-slope",       "slope" },
+};
+
+static const char *annotate_arg_help =
+     "Annotate the output. Comma-separated list of\n"
+     "\t\t\t\t\t\t     gravity, progression, baselines, layout, line,\n"
+     "\t\t\t\t\t\t     run, cluster, char, glyph, caret, slope\n";
+
 static void
 render_callback (PangoLayout *layout,
                  int          x,
@@ -343,8 +369,13 @@ render_callback (PangoLayout *layout,
           iter = pango_layout_get_iter (layout);
           do
             {
+              PangoLayoutRun *run;
               PangoRectangle rect;
 
+              run = pango_layout_iter_get_run (iter);
+              if (!run)
+                continue;
+
               pango_layout_iter_get_run_extents (iter, NULL, &rect);
               cairo_rectangle (cr,
                                (double)rect.x / PANGO_SCALE - lw / 2,
@@ -405,6 +436,176 @@ render_callback (PangoLayout *layout,
           pango_layout_iter_free (iter);
           cairo_restore (cr);
         }
+
+      if (annotate & ANNOTATE_GLYPH_EXTENTS)
+        {
+          /* draw the glyph_extents in blue */
+          cairo_save (cr);
+          cairo_set_source_rgba (cr, 0.0, 0.0, 1.0, 0.5);
+
+          iter = pango_layout_get_iter (layout);
+          do
+            {
+              PangoLayoutRun *run;
+              PangoRectangle rect;
+              int x_pos, y_pos;
+
+              run = pango_layout_iter_get_run (iter);
+              if (!run)
+                continue;
+
+              pango_layout_iter_get_run_extents (iter, NULL, &rect);
+
+              x_pos = rect.x;
+              y_pos = pango_layout_iter_get_run_baseline (iter);
+
+              for (int i = 0; i < run->glyphs->num_glyphs; i++)
+                {
+                  PangoRectangle extents;
+
+                  pango_font_get_glyph_extents (run->item->analysis.font,
+                                                run->glyphs->glyphs[i].glyph,
+                                                &extents, NULL);
+
+                  rect.x = x_pos + run->glyphs->glyphs[i].geometry.x_offset + extents.x;
+                  rect.y = y_pos + run->glyphs->glyphs[i].geometry.y_offset + extents.y;
+                  rect.width = extents.width;
+                  rect.height = extents.height;
+
+                  cairo_rectangle (cr,
+                                   (double)rect.x / PANGO_SCALE - lw / 2,
+                                   (double)rect.y / PANGO_SCALE - lw / 2,
+                                   (double)rect.width / PANGO_SCALE + lw,
+                                   (double)rect.height / PANGO_SCALE + lw);
+                  cairo_stroke (cr);
+
+                  cairo_arc (cr,
+                             (double) (x_pos + run->glyphs->glyphs[i].geometry.x_offset) / PANGO_SCALE,
+                             (double) (y_pos + run->glyphs->glyphs[i].geometry.y_offset) / PANGO_SCALE,
+                             3.0, 0, 2*G_PI);
+                  cairo_fill (cr);
+
+                  x_pos += run->glyphs->glyphs[i].geometry.width;
+                }
+            }
+          while (pango_layout_iter_next_run (iter));
+          pango_layout_iter_free (iter);
+          cairo_restore (cr);
+        }
+
+      if (annotate & ANNOTATE_CARET_POSITIONS)
+        {
+          const PangoLogAttr *attrs;
+          int n_attrs;
+          int offset;
+          int num = 0;
+
+          /* draw the caret positions in purple */
+          cairo_save (cr);
+          cairo_set_source_rgba (cr, 1.0, 0.0, 1.0, 0.5);
+
+          attrs = pango_layout_get_log_attrs_readonly (layout, &n_attrs);
+
+          iter = pango_layout_get_iter (layout);
+          do
+            {
+              PangoRectangle rect;
+              PangoLayoutRun *run;
+              const char *text, *start, *p;
+              int x, y;
+              gboolean trailing;
+
+              pango_layout_iter_get_run_extents (iter, NULL, &rect);
+              run = pango_layout_iter_get_run_readonly (iter);
+
+              if (!run)
+                continue;
+
+              text = pango_layout_get_text (layout);
+              start = text + run->item->offset;
+
+              offset = g_utf8_strlen (text, start - text);
+
+              y = pango_layout_iter_get_run_baseline (iter);
+
+              trailing = FALSE;
+              p = start;
+              for (int i = 0; i <= run->item->num_chars; i++)
+                {
+                  if (attrs[offset + i].is_cursor_position)
+                    {
+                      pango_glyph_string_index_to_x_full (run->glyphs,
+                                                          text + run->item->offset,
+                                                          run->item->length,
+                                                          &run->item->analysis,
+                                                          (PangoLogAttr *)attrs + offset,
+                                                          p - start,
+                                                          trailing,
+                                                          &x);
+                      x += rect.x;
+
+                      cairo_set_source_rgba (cr, 1.0, 0.0, 1.0, 0.5);
+                      cairo_arc (cr, x / PANGO_SCALE, y / PANGO_SCALE, 3.0, 0, 2*G_PI);
+                      cairo_close_path (cr);
+                      cairo_fill (cr);
+
+                      char *s = g_strdup_printf ("%d", num);
+                      cairo_set_source_rgb (cr, 0, 0, 0);
+                      cairo_move_to (cr, x / PANGO_SCALE - 5, y / PANGO_SCALE + 15);
+                      cairo_show_text (cr, s);
+                      g_free (s);
+                   }
+
+                  if (i < run->item->num_chars)
+                    {
+                      num++;
+                      p = g_utf8_next_char (p);
+                    }
+                  else
+                    trailing = TRUE;
+
+                }
+            }
+          while (pango_layout_iter_next_run (iter));
+          pango_layout_iter_free (iter);
+          cairo_restore (cr);
+        }
+
+      if (annotate & ANNOTATE_CARET_SLOPE)
+        {
+          const char *text = pango_layout_get_text (layout);
+          int length = g_utf8_strlen (text, -1);
+          const PangoLogAttr *attrs;
+          int n_attrs;
+          const char *p;
+          int i;
+
+          /* draw the caret slope in gray */
+          cairo_save (cr);
+          cairo_set_source_rgba (cr, 0.0, 0.0, 0.0, 0.5);
+
+          attrs = pango_layout_get_log_attrs_readonly (layout, &n_attrs);
+
+          for (i = 0, p = text; i <= length; i++, p = g_utf8_next_char (p))
+            {
+              PangoRectangle rect;
+
+              if (!attrs[i].is_cursor_position)
+                continue;
+
+              pango_layout_get_caret_pos (layout, p - text, &rect, NULL);
+
+              cairo_move_to (cr,
+                             (double)rect.x / PANGO_SCALE + (double)rect.width / PANGO_SCALE - lw / 2,
+                             (double)rect.y / PANGO_SCALE - lw / 2);
+              cairo_line_to (cr,
+                             (double)rect.x / PANGO_SCALE - lw / 2,
+                             (double)rect.y / PANGO_SCALE + (double)rect.height / PANGO_SCALE - lw / 2);
+              cairo_stroke (cr);
+            }
+
+          cairo_restore (cr);
+        }
     }
 
   cairo_move_to (cr, 0, 0);
@@ -545,21 +746,56 @@ pangocairo_view_display (gpointer instance,
 					   state);
 }
 
+static gboolean
+parse_annotate_arg (const char  *option_name,
+                    const char  *value,
+                    gpointer     data,
+                    GError     **error)
+{
+  guint64 num;
+
+  if (!g_ascii_string_to_unsigned (value, 10, 0, ANNOTATE_LAST - 1, &num, NULL))
+    {
+      char **parts;
+      int i, j;
+
+      parts = g_strsplit (value, ",", 0);
+      num = 0;
+      for (i = 0; parts[i]; i++)
+        {
+          for (j = 0; j < G_N_ELEMENTS (annotate_options); j++)
+            {
+              if (strcmp (parts[i], annotate_options[j].name) == 0 ||
+                  strcmp (parts[i], annotate_options[j].short_name) == 0)
+                {
+                  num |= annotate_options[j].value;
+                  break;
+                }
+            }
+
+          if (j == G_N_ELEMENTS (annotate_options))
+            {
+              g_set_error (error,
+                           G_OPTION_ERROR, G_OPTION_ERROR_FAILED,
+                           "%s is not an allowed value for %s. "
+                           "See --help-cairo", parts[i], option_name);
+              return FALSE;
+            }
+        }
+
+      g_strfreev (parts);
+    }
+
+  opt_annotate = num;
+  return TRUE;
+}
+
 static GOptionGroup *
 pangocairo_view_get_option_group (const PangoViewer *klass G_GNUC_UNUSED)
 {
   GOptionEntry entries[] =
   {
-    {"annotate", 0, 0, G_OPTION_ARG_INT, &opt_annotate,
-     "Annotate the output\n"
-     "\t\t\t\t\t\t\t   1 - gravity\n"
-     "\t\t\t\t\t\t\t   2 - block progression\n"
-     "\t\t\t\t\t\t\t   4 - baselines\n"
-     "\t\t\t\t\t\t\t   8 - layout extents\n"
-     "\t\t\t\t\t\t\t  16 - line extents\n"
-     "\t\t\t\t\t\t\t  32 - run extents\n"
-     "\t\t\t\t\t\t\t  64 - cluster extents\n"
-     "\t\t\t\t\t\t\t 128 - char extents", "FLAGS"},
+    {"annotate", 0, 0, G_OPTION_ARG_CALLBACK, parse_annotate_arg, annotate_arg_help, "FLAGS"},
     {NULL}
   };
   GOptionGroup *group;
